<?xml version="1.0" encoding="UTF-8"?>

<!--/*******************************************************************************
     * Copyright (C) Fuji Xerox Co., Ltd. 2010-2012 All rights reserved.
     *******************************************************************************/-->

<!--GMFモデルからDCASEへの変換XSL-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
                xmlns:dcase_gmf="http://www.dependable-os.net/2010/03/dcase/"
                xmlns:dcase="http://www.dependable-os.net/2010/06/dcase"
                xmlns:exsl="http://exslt.org/common"
                extension-element-prefixes="exsl"
                version="1.0">

  <xsl:output method="xml" encoding="UTF-8" indent="yes"/>

  <!--ルート-->
  <xsl:template match="/">
    <xsl:element name="dcase:dcase" namespace="http://www.dependable-os.net/2010/06/dcase">
      <xsl:apply-templates select="dcase_gmf:Argument"/>
    </xsl:element>
  </xsl:template>

  <xsl:template match="dcase_gmf:Argument">
    <!-- リンクをルートからたどった結果セット -->
    <xsl:variable name="linkResult">
      <xsl:for-each select="rootBasicNode">
        <xsl:variable name="id" select="concat('#',@id)"/>
        <xsl:variable name="count" select="count(/dcase_gmf:Argument/rootBasicLink[@target=$id])"></xsl:variable>
        <xsl:if test="$count=0">
          <xsl:call-template name="RootBasicLink">
            <xsl:with-param name="source" select="$id"/>
          </xsl:call-template>
        </xsl:if>
      </xsl:for-each>
    </xsl:variable>

    <xsl:call-template name="ArgumentAttribute"/>
    <xsl:element name="dcase:description" namespace="http://www.dependable-os.net/2010/06/dcase">
      <xsl:value-of select="@desc"/>
    </xsl:element>
    <xsl:call-template name="Properties"/>
    <xsl:element name="dcase:nodes" namespace="http://www.dependable-os.net/2010/06/dcase">
      <xsl:apply-templates select="rootBasicNode"/>
    </xsl:element>
    <xsl:element name="dcase:links" namespace="http://www.dependable-os.net/2010/06/dcase">
      <!-- ルートからたどれるリンクを重複を省いて出力 -->
      <xsl:for-each select="exsl:node-set($linkResult)/dcase:link[not(@id=preceding::dcase:link/@id)]">
        <xsl:copy-of select="."/>
      </xsl:for-each>

      <!-- ルートからたどれないリンク（リング状）を出力 -->
      <xsl:apply-templates select="rootBasicLink[not(@id=exsl:node-set($linkResult)/dcase:link/@id)]" mode="basic"></xsl:apply-templates>
    </xsl:element>
  </xsl:template>

  <!--ノード-->
  <xsl:template match="rootBasicNode">
    <xsl:element name="dcase:node" namespace="http://www.dependable-os.net/2010/06/dcase">
      <xsl:call-template name="BasicNodeAttribute"/>
      <xsl:element name="dcase:description" namespace="http://www.dependable-os.net/2010/06/dcase">
        <xsl:value-of select="@desc"/>
      </xsl:element>
      <xsl:call-template name="Properties"/>
    </xsl:element>
  </xsl:template>

  <!--リンク-->

  <xsl:template name="RootBasicLink">
    <xsl:param name="ancestors"></xsl:param>
    <xsl:param name="source"></xsl:param>

    <!-- サイクリックをチェックするためにAncestorを保持する -->
    <xsl:variable name="tmpAncestors">
      <xsl:copy-of select="$ancestors"/>
    </xsl:variable>

    <xsl:variable name="newAncestors">
      <xsl:copy-of select="$ancestors"/>
      <AncestorID>
        <xsl:value-of select="$source"/>
      </AncestorID>
    </xsl:variable>

    <!--再帰チェック-->
    <xsl:if test="not(exsl:node-set($tmpAncestors)/AncestorID[.=$source])">
      <xsl:apply-templates select="/dcase_gmf:Argument/rootBasicLink[(@source=$source) and (string(number(@userdef001)) != 'NaN')]" mode="basic">
        <xsl:sort select="number(@userdef001)" data-type="number" order="ascending"/>
      </xsl:apply-templates>
      <xsl:apply-templates select="/dcase_gmf:Argument/rootBasicLink[(@source=$source) and (string(number(@userdef001)) = 'NaN')]" mode="basic"/>

      <xsl:apply-templates select="/dcase_gmf:Argument/rootBasicLink[(@source=$source) and (string(number(@userdef001)) != 'NaN')]" mode="procChildren">
        <xsl:sort select="./@userdef001" data-type="number" order="ascending"/>
        <xsl:with-param name="ancestors" select="$newAncestors"/>
      </xsl:apply-templates>
      <xsl:apply-templates select="/dcase_gmf:Argument/rootBasicLink[(@source=$source) and (string(number(@userdef001)) = 'NaN')]" mode="procChildren">
        <xsl:with-param name="ancestors" select="$newAncestors"/>
      </xsl:apply-templates>
    </xsl:if>
  </xsl:template>

  <!-- リンク要素出力 -->
  <xsl:template match="rootBasicLink" mode="basic">
    <xsl:element name="dcase:link" namespace="http://www.dependable-os.net/2010/06/dcase">
      <xsl:call-template name="BasicLinkAttribute"/>
      <xsl:element name="dcase:description" namespace="http://www.dependable-os.net/2010/06/dcase">
        <xsl:value-of select="@desc"/>
      </xsl:element>
      <xsl:call-template name="Properties"/>
    </xsl:element>
  </xsl:template>

  <xsl:template match="rootBasicLink" mode="procChildren">
    <xsl:param name="ancestors"></xsl:param>
    <xsl:call-template name="RootBasicLink">
      <xsl:with-param name="ancestors" select="$ancestors"/>
      <xsl:with-param name="source" select="./@target"/>
    </xsl:call-template>
  </xsl:template>


  <!--ルートの属性設定-->
  <xsl:template name="ArgumentAttribute">
    <xsl:apply-templates select="@id|@name|@status" mode="basic"/>
  </xsl:template>


  <!--ノードの属性設定-->
  <xsl:template name="BasicNodeAttribute">
    <xsl:call-template name="NodeType"/>
    <xsl:apply-templates select="@id|@name|@status" mode="basic"/>
  </xsl:template>

  <!--ノード種別設定-->
  <xsl:template name="NodeType">
    <xsl:attribute name="type">
      <xsl:choose>
        <xsl:when test="@xsi:type='dcase:Goal'">Goal</xsl:when>
        <xsl:when test="@xsi:type='dcase:Strategy'">Strategy</xsl:when>
        <xsl:when test="@xsi:type='dcase:Evidence'">Evidence</xsl:when>
        <xsl:when test="@xsi:type='dcase:Undeveloped'">Undeveloped</xsl:when>
        <xsl:when test="@xsi:type='dcase:Context'">Context</xsl:when>
        <xsl:when test="@xsi:type='dcase:Monitor'">Monitor</xsl:when>
        <xsl:when test="@xsi:type='dcase:Justification'">Justification</xsl:when>
        <xsl:when test="@xsi:type='dcase:System'">System</xsl:when>
        <xsl:when test="@xsi:type='dcase:Policy'">Policy</xsl:when>
        <xsl:when test="@xsi:type='dcase:Userdef001'">Userdef001</xsl:when>
        <xsl:when test="@xsi:type='dcase:Userdef002'">Userdef002</xsl:when>
        <xsl:when test="@xsi:type='dcase:Userdef003'">Userdef003</xsl:when>
        <xsl:when test="@xsi:type='dcase:Userdef004'">Userdef004</xsl:when>
        <xsl:when test="@xsi:type='dcase:Userdef005'">Userdef005</xsl:when>
        <xsl:when test="@xsi:type='dcase:Userdef006'">Userdef006</xsl:when>
        <xsl:otherwise>undefined</xsl:otherwise>
      </xsl:choose>
    </xsl:attribute>
  </xsl:template>

  <!--リンクの属性設定-->
  <xsl:template name="BasicLinkAttribute">
    <xsl:apply-templates select="@id|@name|@status" mode="basic"/>
    <xsl:apply-templates select="@source|@target" mode="link"/>
  </xsl:template>

  <!--属性設定-->
  <xsl:template match="@*" mode="basic">
    <xsl:attribute name="{local-name()}">
      <xsl:value-of select="."/>
    </xsl:attribute>
  </xsl:template>

  <!--リンクのsourceとtarget-->
  <xsl:template match="@*" mode="link">
    <xsl:attribute name="{local-name()}">
      <xsl:value-of select="substring-after(.,'#') "/>
    </xsl:attribute>
  </xsl:template>

  <!--プロパティ設定-->
  <xsl:template name="Properties">
    <xsl:element name="dcase:properties" namespace="http://www.dependable-os.net/2010/06/dcase">
      <xsl:apply-templates select="@attachment" mode="property">
        <xsl:with-param name="propertyName">Attachment</xsl:with-param>
      </xsl:apply-templates>
      <xsl:apply-templates select="@userdef001" mode="property">
        <xsl:with-param name="propertyName">Userdef001</xsl:with-param>
      </xsl:apply-templates>
      <xsl:apply-templates select="@userdef002" mode="property">
        <xsl:with-param name="propertyName">Userdef002</xsl:with-param>
      </xsl:apply-templates>
      <xsl:apply-templates select="@userdef003" mode="property">
        <xsl:with-param name="propertyName">Userdef003</xsl:with-param>
      </xsl:apply-templates>
      <xsl:apply-templates select="@userdef004" mode="property">
        <xsl:with-param name="propertyName">Userdef004</xsl:with-param>
      </xsl:apply-templates>
      <xsl:apply-templates select="@userdef005" mode="property">
        <xsl:with-param name="propertyName">Userdef005</xsl:with-param>
      </xsl:apply-templates>
      <xsl:apply-templates select="@userdef006" mode="property">
        <xsl:with-param name="propertyName">Userdef006</xsl:with-param>
      </xsl:apply-templates>
      <xsl:apply-templates select="@userdef007" mode="property">
        <xsl:with-param name="propertyName">Userdef007</xsl:with-param>
      </xsl:apply-templates>
      <xsl:apply-templates select="@userdef008" mode="property">
        <xsl:with-param name="propertyName">Userdef008</xsl:with-param>
      </xsl:apply-templates>
      <xsl:apply-templates select="@userdef009" mode="property">
        <xsl:with-param name="propertyName">Userdef009</xsl:with-param>
      </xsl:apply-templates>
      <xsl:apply-templates select="@userdef010" mode="property">
        <xsl:with-param name="propertyName">Userdef010</xsl:with-param>
      </xsl:apply-templates>
      <xsl:apply-templates select="@userdef011" mode="property">
        <xsl:with-param name="propertyName">Userdef011</xsl:with-param>
      </xsl:apply-templates>
      <xsl:apply-templates select="@userdef012" mode="property">
        <xsl:with-param name="propertyName">Userdef012</xsl:with-param>
      </xsl:apply-templates>
      <xsl:apply-templates select="@userdef013" mode="property">
        <xsl:with-param name="propertyName">Userdef013</xsl:with-param>
      </xsl:apply-templates>
      <xsl:apply-templates select="@userdef014" mode="property">
        <xsl:with-param name="propertyName">Userdef014</xsl:with-param>
      </xsl:apply-templates>
      <xsl:apply-templates select="@userdef015" mode="property">
        <xsl:with-param name="propertyName">Userdef015</xsl:with-param>
      </xsl:apply-templates>
      <xsl:apply-templates select="@userdef016" mode="property">
        <xsl:with-param name="propertyName">Userdef016</xsl:with-param>
      </xsl:apply-templates>
      <xsl:apply-templates select="@score" mode="property">
        <xsl:with-param name="propertyName">Score</xsl:with-param>
      </xsl:apply-templates>
      <xsl:apply-templates select="@weight" mode="property">
        <xsl:with-param name="propertyName">Weight</xsl:with-param>
      </xsl:apply-templates>
      <xsl:apply-templates select="@nodeLink" mode="property">
        <xsl:with-param name="propertyName">NodeLink</xsl:with-param>
      </xsl:apply-templates>
      <xsl:apply-templates select="@isNormal" mode="property">
        <xsl:with-param name="propertyName">IsNormal</xsl:with-param>
      </xsl:apply-templates>
      <xsl:apply-templates select="@stakeholder" mode="property">
        <xsl:with-param name="propertyName">Stakeholder</xsl:with-param>
      </xsl:apply-templates>
      <xsl:apply-templates select="@riskAnalysis" mode="property">
        <xsl:with-param name="propertyName">RiskAnalysis</xsl:with-param>
      </xsl:apply-templates>
    </xsl:element>
  </xsl:template>


  <!--各属性値を設定-->
  <xsl:template match="@*" mode="property">
    <xsl:param name="propertyName">undefined</xsl:param>
    <xsl:element name="dcase:property" namespace="http://www.dependable-os.net/2010/06/dcase">
      <xsl:attribute name="name">
        <xsl:value-of select="$propertyName"/>
      </xsl:attribute>
      <xsl:attribute name="value">
        <xsl:value-of select="."/>
      </xsl:attribute>
    </xsl:element>
  </xsl:template>

</xsl:stylesheet>
